#!/bin/bash
# © Copyright [2020, 2022, 2024] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

# Log files
TMP_LOG_FILE="/tmp/cis_upgrade.log"
FILE_NAME="cis_upgrade.sh"
DATE_NOW=$(date +%s)
LOG_FILE="/var/opt/novell/log/cis/cis_upgrade_$DATE_NOW.log"

usage(){
cat << EOF
CIS upgrade utility

Usage: cis_upgrade.sh <OPTIONS>

GENERAL OPTIONS (OPTIONS):
  -v, --version         Displays the program version information.
  -o, --opensearch      Moves ZooKeeper data to OpenSearch.
  -h, --help            Displays the help information.

This utility will upgrade CIS to CIS SP 3.0
If CIS files are already stored on media, they will continue to be on media after the upgrade.
If CIS configuration and log files were never moved to media, this utility will treat "/" as media.
EOF
}

version(){
  # Get Server version.
  oes_version=`grep "VERSION" /etc/novell-release | cut -d'=' -f2- | xargs`
  echo -e "CIS upgrade utility version $oes_version"
}

printandExit(){
	if [[ $1 -ne 0 ]]; then
    echo -e "CIS service upgradation failed. Check /var/opt/novell/log/cis/cis_upgrade*.log"
		echo -e "CIS service upgradation failed." | tee -a "$TMP_LOG_FILE"
    mv "$TMP_LOG_FILE" "$LOG_FILE"
		exit 1
	else
    echo -e "CIS service upgrade completed successfully. Check /var/opt/novell/log/cis/cis_upgrade*.log " 
		echo -e "CIS service upgrade completed successfully." | tee -a "$TMP_LOG_FILE"
    mv "$TMP_LOG_FILE" "$LOG_FILE"
		exit 0
	fi
}

isStringPresent() {
  line=$1
  file=$2
  notfound=`grep "$line" "$file"`
  if [ -z "$notfound" ]; then
    return 0
  else
    return 1
  fi
}

stopServices(){
    # Stopping all CIS core services, If running
    declare -a cis_service_names=(
        "gateway" "mgmt"
        "collector" "policy"
        "repcollector" "aggregator"
        "metadata" "auth"
        "data" "repaggregator"           
    )

    for name in "${cis_service_names[@]}"
    do
      echo -e "Stopping oes-cis-$name.service." | tee -a "$TMP_LOG_FILE"
      systemctl stop oes-cis-"$name".service

      if [ $? -ne 0 ]; then
        echo -e "Unable to stop oes-cis-$name.service." | tee -a "$TMP_LOG_FILE"
        printandExit 1
      fi

    done
}

backupCertificates(){
  echo -e "Backing up existing CIS certificates." | tee -a "$TMP_LOG_FILE"
  if ! { [ -e /etc/opt/novell/cis/backup/certs/servercert.pem ] && [ -e /etc/opt/novell/cis/backup/certs/serverkey.pem ] && [ -e /etc/opt/novell/cis/backup/certs/rootCAs/SSCert.pem ]; }; then
    mv /etc/opt/novell/cis/certs/servercert.pem /etc/opt/novell/cis/backup/certs
    mv /etc/opt/novell/cis/certs/serverkey.pem /etc/opt/novell/cis/backup/certs
    mv /etc/opt/novell/cis/certs/rootCAs/SSCert.pem /etc/opt/novell/cis/backup/certs/rootCAs
  fi

  cp /etc/ssl/servercerts/servercert.pem /etc/opt/novell/cis/certs
  cp /etc/ssl/servercerts/serverkey.pem /etc/opt/novell/cis/certs
  cp /etc/opt/novell/certs/SSCert.pem /etc/opt/novell/cis/certs/rootCAs
  
  /opt/novell/cis/bin/appendcert -dir /etc/opt/novell/certs/ -leaf /etc/ssl/servercerts/servercert.pem -chain /etc/opt/novell/cis/certs/rootCAs/SSCert.pem

  ## Fix Kafka certificates
  echo -e "Importing certs in kafka key store." | tee -a "$TMP_LOG_FILE"
  KAFKA_ADV_ADDR=$(grep "MQ_ADVERTISED_ADDRESS" /etc/opt/novell/cis/kafka/config | cut -d'=' -f2- | xargs)
  if [[ $KAFKA_ADV_ADDR != "" ]]; then
    if ! { [ -e /etc/opt/novell/cis/certs/servercert.pem ] && [ -e /etc/opt/novell/cis/certs/serverkey.pem ] && [ -e /etc/opt/novell/cis/certs/rootCAs/SSCert.pem ]; }; then
      echo "Server certificates not found"  | tee -a "$TMP_LOG_FILE"
      printandExit 1;
    fi

    if ! { [ -e /etc/opt/novell/cis/kafka/certs/server.truststore.jks ] && [ -e /etc/opt/novell/cis/kafka/certs/server.keystore.jks ] && [ -e /etc/opt/novell/cis/kafka/certs/server.p12 ]; }; then
      echo "Kafka certificates not found"  | tee -a "$TMP_LOG_FILE"
      config_file=/etc/opt/novell/cis/kafka/config
      password=$(head /dev/urandom | tr -dc A-Za-z0-9 | head -c 13 ; echo '')
      rm -f /etc/opt/novell/cis/kafka/certs/*

      openssl pkcs12 -export -in /etc/opt/novell/cis/certs/servercert.pem -inkey /etc/opt/novell/cis/certs/serverkey.pem -out /etc/opt/novell/cis/kafka/certs/server.p12 -name localhost -CAfile /etc/opt/novell/cis/certs/rootCAs/SSCert.pem -caname rootca -password pass:"$password";  >> "$TMP_LOG_FILE" 2>&1
      if [ $? -ne 0 ]; then
        echo "Failed to import the server certificate, private key and CA certificate into PKCS12 keystore"  | tee -a "$TMP_LOG_FILE"
        printandExit 1;
      fi

      /opt/novell/cis/bin/cismigrate --javaversion
      if [[ $? -eq 0 ]]; then
        keytool -J-Dkeystore.pkcs12.legacy -importkeystore -deststorepass "$password" -destkeypass "$password" -destkeystore /etc/opt/novell/cis/kafka/certs/kafka.keystore.jks -srckeystore /etc/opt/novell/cis/kafka/certs/server.p12 -srcstoretype PKCS12 -srcstorepass "$password" -alias localhost; >> "$TMP_LOG_FILE" 2>&1
        if [ $? -ne 0 ]; then
          echo "Failed to Covert the PKCS12 type keystore to JKS type keystore"  | tee -a "$TMP_LOG_FILE"
          printandExit 1;
        fi

        keytool -J-Dkeystore.pkcs12.legacy -keystore /etc/opt/novell/cis/kafka/certs/kafka.truststore.jks -storepass "$password" -alias CARoot -import -file /etc/opt/novell/cis/certs/rootCAs/SSCert.pem -noprompt >> "$TMP_LOG_FILE" 2>&1
        if [ $? -ne 0 ]; then
          echo "Failed to write MQ_KEYSTORE_PLAINTEXT_PASSWORD in kafka config file"  | tee -a "$TMP_LOG_FILE"
          printandExit 1;
        fi
      else
        keytool -importkeystore -deststorepass "$password" -destkeypass "$password" -destkeystore /etc/opt/novell/cis/kafka/certs/kafka.keystore.jks -srckeystore /etc/opt/novell/cis/kafka/certs/server.p12 -srcstoretype PKCS12 -srcstorepass "$password" -alias localhost; >> "$TMP_LOG_FILE" 2>&1
        if [ $? -ne 0 ]; then
          echo "Failed to Covert the PKCS12 type keystore to JKS type keystore"  | tee -a "$TMP_LOG_FILE"
          printandExit 1;
        fi

        keytool -keystore /etc/opt/novell/cis/kafka/certs/kafka.truststore.jks -storepass "$password" -alias CARoot -import -file /etc/opt/novell/cis/certs/rootCAs/SSCert.pem -noprompt >> "$TMP_LOG_FILE" 2>&1
        if [ $? -ne 0 ]; then
          echo "Failed to write MQ_KEYSTORE_PLAINTEXT_PASSWORD in kafka config file"  | tee -a "$TMP_LOG_FILE"
          printandExit 1;
        fi
      fi
      sed -i -e"s/^MQ_KEYSTORE_PLAINTEXT_PASSWORD=.*/MQ_KEYSTORE_PLAINTEXT_PASSWORD=\"$password\"/" "$config_file"
      sed -i -e"s/^MQ_KEY_PLAINTEXT_PASSWORD=.*/MQ_KEY_PLAINTEXT_PASSWORD=\"$password\"/" "$config_file"
      sed -i -e"s/^MQ_TRUSTSTORE_PLAINTEXT_PASSWORD=.*/MQ_TRUSTSTORE_PLAINTEXT_PASSWORD=\"$password\"/" "$config_file"
    else
      mv /etc/opt/novell/cis/kafka/certs/server.truststore.jks /etc/opt/novell/cis/kafka/certs/kafka.truststore.jks
      mv /etc/opt/novell/cis/kafka/certs/server.keystore.jks /etc/opt/novell/cis/kafka/certs/kafka.keystore.jks
    fi
  fi
}

stopCISServices() {
  # Stopping all CIS core services, If running
  declare -a cis_service_names=(
      "gateway" "mgmt" 
      "collector" "policy" 
      "repcollector" "aggregator" 
      "metadata" "auth" 
      "data" "repaggregator"
      "gatewayforscale" "dataatscale"
      "configuration"
  )
  for name in "${cis_service_names[@]}"
  do
      echo -e "Stopping oes-cis-$name.service." | tee -a "$TMP_LOG_FILE"
      systemctl stop oes-cis-"$name".service >> "$TMP_LOG_FILE" 2>&1
  done

  echo -e "Stopping oes-dashboard.service." | tee -a "$TMP_LOG_FILE"
  systemctl stop oes-dashboard.service >> "$TMP_LOG_FILE" 2>&1
}

updateDockerImages() {
  docker stop cis-fluentbit >> "$TMP_LOG_FILE" 2>&1

  count=`docker ps -a -f "status=created" -f "status=exited" -f "status=dead" -q`;
  if [[ $count != "" ]]; then
      docker rm $(docker ps -a -f "status=created" -f "status=exited" -f "status=dead" -q) >> "$TMP_LOG_FILE" 2>&1
  fi

  echo -e "Updating CIS service docker images." | tee -a "$TMP_LOG_FILE"
  declare -a service_names=(
      "gateway" "mgmt"
      "collector" "policy"
      "repcollector" "aggregator"
      "metadata" "auth"
      "data" "repaggregator"
  )
  for name in "${service_names[@]}"
  do
      /opt/novell/cis/bin/cis_image_version_update.sh "$name" >> "$TMP_LOG_FILE" 2>&1
  done
  /opt/novell/cis/bin/oes_dashboard_image_version_update.sh >> "$TMP_LOG_FILE" 2>&1 

  systemctl daemon-reload
}

moveToOpensearch() {
  fromMgmt=$1

  if [ "$fromMgmt" == "1" ]; then
    config_Type=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'configType'\042/){print $(i+1)}}}' | tr -d '"' | xargs`
    if [ "$config_Type" == "core" ]; then
      exit 0
    fi
  fi
  
  ZK_URI=`grep "ZK_URI" /etc/opt/novell/cis/config | cut -d'=' -f2- | xargs`

  if { [ "$ZK_URI" == "" ]; };then  
    echo -e "Failed to get ZooKeeper uri from file /etc/opt/novell/cis/config"
    echo -e "Cannot set OpenSearch configuration in ZooKeeper."
  else
    # do not print to log file
    if [ "$fromMgmt" == "1" ]; then
      /opt/novell/cis/bin/cismigrate --opensearch -zk_uri "$ZK_URI" > /dev/null 2>&1 
    else
      /opt/novell/cis/bin/cismigrate --opensearch -zk_uri "$ZK_URI" >> "$TMP_LOG_FILE" 2>&1 
    fi
    if [[ $? -eq 0 ]]; then
      sed -i "s/esURI/osURI/g" /etc/opt/novell/cis/configurationStatus.json
      sed -i "s/esPort/osPort/g" /etc/opt/novell/cis/configurationStatus.json
      echo -e "Opensearch configuration values set in ZooKeeper."      
    else
      echo -e "Opensearch configuration values could not be set in ZooKeeper."      
    fi    
  fi
}  

updateForElastic() {
  ports="19300"
  esfile="/opt/novell/cis/es/bin/es_service.sh"

  isStringPresent $ports $esfile

  if [ $? == 0 ]; then
     sed -i -e"s/9300:9300/19300:9300/g" $file
  fi 
  firewallstate=$(firewall-cmd --state 2>&1)
  echo "${DATE_NOW} - firewall is ${firewallstate} - cis_upgrade.sh" >> "$LOG_FILE"
  if [ "$firewallstate" == "running" ]; then             
    firewallop=$(firewall-cmd --permanent --query-port=9500/tcp) > /dev/null 2>&1
    echo "$DATE_NOW - firewall port:9500/tcp is added: $firewallop - $FILE_NAME" >> "$LOG_FILE"
    if [ "$firewallop" != "yes" ]; then
      firewallop=$(firewall-cmd --add-port=9500/tcp --permanent) > /dev/null 2>&1
      echo "$DATE_NOW - adding firewall port:9500/tcp: $firewallop - $FILE_NAME" >> "$LOG_FILE"
    fi
    firewall-cmd --reload > /dev/null 2>&1
  else
    firewallop=$(firewall-offline-cmd --query-port=9500/tcp) > /dev/null 2>&1
    echo "$DATE_NOW - firewall offline port:9500/tcp is added: $firewallop - $FILE_NAME" >> "$LOG_FILE"
    if [ "$firewallop" != "yes" ]; then
      firewallop=$(firewall-offline-cmd --add-port=9500/tcp) > /dev/null 2>&1
      echo "$DATE_NOW - adding firewall offline port:9500/tcp: $firewallop - $FILE_NAME" >> "$LOG_FILE"
    fi
  fi
   # update config
  sed -i -e"s/9400/9500/g" /etc/opt/novell/cis/es/config
  sed -i -e"s/9400/9500/g" /etc/opt/novell/cis/es/es_envoy.json  
}

upgradeCisStandalone(){    
  stopCISServices
  
  # Stopping all CIS external dependencies, If running
  declare -a cis_ext_service_names=(
      "elastic" "kafka"
      "zk" "fluentbit"
  )
  for name in "${cis_ext_service_names[@]}"
  do
      echo -e "Stopping oes-cis-$name.service." | tee -a "$TMP_LOG_FILE"
      systemctl stop oes-cis-"$name".service >> "$TMP_LOG_FILE" 2>&1
  done

  updateDockerImages

  systemctl enable oes-cis-fluentbit.service >> "$TMP_LOG_FILE" 2>&1    
  backupCertificates

  systemctl is-active --quiet oes-cis-zk.service >> "$TMP_LOG_FILE" 2>&1
  if [[ $? -ne 0 ]]; then
    echo -e "Starting CIS ZooKeeper service."
      systemctl start oes-cis-zk.service >> "$TMP_LOG_FILE" 2>&1
      systemctl is-active --quiet oes-cis-zk.service >> "$TMP_LOG_FILE" 2>&1
      if [[ $? -ne 0 ]]; then
          echo -e "Failed to start CIS ZooKeeper service.\n" | tee -a "$TMP_LOG_FILE"
          printandExit 1
      fi
  fi
  sleep 15
  chmod -R 0776 /etc/opt/novell/cis/kafka/kafka-logs/
  chmod 0777 /etc/opt/novell/cis/kafka/certs/server.p12
  docker rmi wurstmeister/kafka:2.12-2.5.0 >> "$TMP_LOG_FILE" 2>&1
  systemctl is-active --quiet oes-cis-kafka.service >> "$TMP_LOG_FILE" 2>&1
  if [[ $? -ne 0 ]]; then
    echo -e "Starting CIS kafka service. \n" | tee -a "$TMP_LOG_FILE"
      systemctl start oes-cis-kafka.service >> "$TMP_LOG_FILE" 2>&1
      systemctl is-active --quiet oes-cis-kafka.service >> "$TMP_LOG_FILE" 2>&1
      if [[ $? -ne 0 ]]; then
          echo -e "Failed to start CIS kafka service. \n" | tee -a "$TMP_LOG_FILE"
          printandExit 1
      fi
  fi
  sleep 15

  updateForElastic
  systemctl is-active --quiet oes-cis-elastic.service >> "$TMP_LOG_FILE" 2>&1
  if [[ $? -ne 0 ]]; then
    echo -e "Starting CIS elastic service for migration."
      systemctl start oes-cis-elastic.service >> "$TMP_LOG_FILE" 2>&1
      systemctl is-active --quiet oes-cis-elastic.service >> "$TMP_LOG_FILE" 2>&1
      if [[ $? -ne 0 ]]; then
          echo -e "Failed to start CIS elastic service. \n" | tee -a "$TMP_LOG_FILE"
          printandExit 1
      fi
  fi
  sleep 15 
  
  systemctl enable oes-cis-opensearch.service >> "$TMP_LOG_FILE" 2>&1  
  systemctl is-active --quiet oes-cis-opensearch.service >> "$TMP_LOG_FILE" 2>&1
  if [[ $? -ne 0 ]]; then
    echo -e "Starting CIS opensearch service."
      systemctl start oes-cis-opensearch.service >> "$TMP_LOG_FILE" 2>&1
      systemctl is-active --quiet oes-cis-opensearch.service >> "$TMP_LOG_FILE" 2>&1
      if [[ $? -ne 0 ]]; then
          echo -e "Failed to start CIS opensearch service. \n" | tee -a "$TMP_LOG_FILE"
          printandExit 1
      fi
  fi
  sleep 15  

  moveToOpensearch "0"
}

upgradeCisCore(){
  moveToOpensearch "0"
  stopCISServices
  # Removing dangling container images
  docker stop cis-fluentbit >> "$TMP_LOG_FILE" 2>&1
  
  updateDockerImages
  systemctl daemon-reload

  systemctl enable oes-cis-fluentbit.service >> "$TMP_LOG_FILE" 2>&1      
  backupCertificates
}

upgradeScale(){
    # Stopping all CIS core services, If running
    declare -a cis_service_names=(
        "gatewayforscale" "dataatscale"
        "configuration"
    )
    for name in "${cis_service_names[@]}"
    do
	    echo -e "Stopping oes-cis-$name.service. \n" | tee -a "$TMP_LOG_FILE"
        systemctl stop oes-cis-"$name".service >> "$TMP_LOG_FILE" 2>&1
    done

    ZK_URI=`grep "ZK_URI" /etc/opt/novell/cis-scale/config | cut -d'=' -f2- | xargs`
    CIS_SERVER=`grep "CIS_SERVER" /etc/opt/novell/cis-scale/config | cut -d'=' -f2- | xargs`
    /opt/novell/cis/bin/cismigrate --scale -zk_uri "$ZK_URI" -cis_server "$CIS_SERVER"  >> "$TMP_LOG_FILE" 2>&1
    if [[ $? -ne 0 ]]; then
        echo -e "Failed to register CIS scale server. \n" | tee -a "$TMP_LOG_FILE"
        printandExit 1
    fi
    conf='/etc/opt/novell/cis-scale/config'
    grep "^CIS_NET" $conf
    if [[ $? -ne 0 ]]; then
        echo -e "\nCIS_NET=\"cisnet\"" >> "$conf"
    fi
    sh /opt/novell/cis/bin/activate_cis_scale.sh  >> "$TMP_LOG_FILE" 2>&1        
}

updateConfiguration() {  
  [[ ! -d /etc/opt/novell/cis/gateway ]] && cp -rp /etc/opt/novell/cis_local/gateway /etc/opt/novell/cis/
  [[ ! -f /etc/opt/novell/cis/cis3.yml ]] && cp -p /etc/opt/novell/cis_local/cis3.yml /etc/opt/novell/cis/cis3.yml
  [[ ! -f /etc/opt/novell/cis/cis5.yml ]] && cp -p /etc/opt/novell/cis_local/cis5.yml /etc/opt/novell/cis/cis5.yml
  [[ ! -f /etc/opt/novell/cis/cis7.yml ]] && cp -p /etc/opt/novell/cis_local/cis7.yml /etc/opt/novell/cis/cis7.yml
  [[ ! -f /etc/opt/novell/cis/status.json ]] && cp -p /etc/opt/novell/cis_local/status.json /etc/opt/novell/cis/status.json

}

# Entry Point
while [ "$1" != "" ]; do
  case $1 in
    --help )             usage
                         exit 0
                         ;;
    -h )                 usage
                         exit 0
                         ;;
    --version )          version
                         exit 0
                         ;;
    -v )                 version
                         exit 0
                         ;;
    --opensearch )       moveToOpensearch "1"
                         exit 0
                         ;;
    -o )                 moveToOpensearch "1"
                         exit 0
                         ;; 
    * )                  usage
                         exit 1
                         ;;
    esac
                         shift
done

#check if links are not broken
if { [ -L /etc/opt/novell/cis ] && [ ! -e /etc/opt/novell/cis ]; };then
  echo -e "NSS resource not mounted on the current node." | tee -a "$TMP_LOG_FILE"
  printandExit 1       
fi

prefix_path=""
is_Configured=""

#Todo prefix path should be non empty for sp2.0 or if any updates have happened
#media should be mounted else flag an error

is_Configured=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'cisConfigSuccess'\042/){print $(i+1)}}}' | tr -d '"' | xargs`
prefix_path=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'prefixPath'\042/){print $(i+1)}}}' | tr -d '"' | xargs`
echo -e "CIS prefix path is $prefix_path" | tee -a "$TMP_LOG_FILE"

#copy all new configuration to the attached cis now
if [ -f /etc/opt/novell/cis/nginx.conf ]; then
  rm /etc/opt/novell/cis/nginx.conf
fi

if [ -e /etc/opt/novell/cis/kafka/config ]; then
  kf_conf='/etc/opt/novell/cis/kafka/config'
  sed -i -e"s/^MQ_KEYSTORE_FILENAME=.*/MQ_KEYSTORE_FILENAME=\"kafka.keystore.jks\"/" "$kf_conf"
  sed -i -e"s/^MQ_TRUSTSTORE_FILENAME=.*/MQ_TRUSTSTORE_FILENAME=\"kafka.truststore.jks\"/" "$kf_conf"
fi

/usr/bin/sh /opt/novell/cis/bin/copy_upgraded_files.sh "$prefix_path"

# Get configuration status and type.
config_Type=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'configType'\042/){print $(i+1)}}}' | tr -d '"' | xargs`

echo -e "Beginning CIS service upgrade." | tee -a "$TMP_LOG_FILE"
echo -e "CIS services configured in $config_Type mode" | tee -a "$TMP_LOG_FILE"
echo -e "Configuration path: $prefix_path" | tee -a "$TMP_LOG_FILE"

/sbin/sysctl --system 2>&1 > /dev/null


if { [ "$config_Type" == "standalone" ]; };then    
    upgradeCisStandalone 
fi

if { [ "$config_Type" == "core" ]; };then
    upgradeCisCore 
fi

if { [ "$config_Type" == "cluster" ]; };then
    upgradeCisCore 
fi

if { [ "$config_Type" == "scale" ]; };then
    upgradeScale
fi

printandExit 0